@@ -522,7 +522,7 @@ static int ioplug_drain_via_poll(snd_pcm_t *pcm)
 		/* in non-blocking mode, let application to poll() by itself */
 		if (io->data->nonblock)
 			return -EAGAIN;
-		if (snd_pcm_wait_nocheck(pcm, -1) < 0)
+		if (snd_pcm_wait_nocheck(pcm, SND_PCM_WAIT_DRAIN) < 0)
 			break;
 	}
 
