@@ -400,6 +400,14 @@ int uc_mgr_config_load(int format, const char *file, snd_config_t **cfg)
 	return 0;
 }
 
+static void uc_mgr_free_value1(struct ucm_value *val)
+{
+	free(val->name);
+	free(val->data);
+	list_del(&val->list);
+	free(val);
+}
+
 void uc_mgr_free_value(struct list_head *base)
 {
 	struct list_head *pos, *npos;
@@ -407,10 +415,7 @@ void uc_mgr_free_value(struct list_head *base)
 	
 	list_for_each_safe(pos, npos, base) {
 		val = list_entry(pos, struct ucm_value, list);
-		free(val->name);
-		free(val->data);
-		list_del(&val->list);
-		free(val);
+		uc_mgr_free_value1(val);
 	}
 }
 
@@ -515,6 +520,10 @@ void uc_mgr_free_sequence_element(struct sequence_element *seq)
 	case SEQUENCE_ELEMENT_TYPE_CFGSAVE:
 		free(seq->data.cfgsave);
 		break;
+	case SEQUENCE_ELEMENT_TYPE_DEV_ENABLE_SEQ:
+	case SEQUENCE_ELEMENT_TYPE_DEV_DISABLE_SEQ:
+		free(seq->data.device);
+		break;
 	default:
 		break;
 	}
@@ -704,11 +713,35 @@ int uc_mgr_set_variable(snd_use_case_mgr_t *uc_mgr, const char *name,
 	return 0;
 }
 
+int uc_mgr_delete_variable(snd_use_case_mgr_t *uc_mgr, const char *name)
+{
+	struct list_head *pos;
+	struct ucm_value *curr;
+
+	list_for_each(pos, &uc_mgr->variable_list) {
+		curr = list_entry(pos, struct ucm_value, list);
+		if (strcmp(curr->name, name) == 0) {
+			uc_mgr_free_value1(curr);
+			return 0;
+		}
+	}
+
+	return -ENOENT;
+}
+
 void uc_mgr_free_verb(snd_use_case_mgr_t *uc_mgr)
 {
 	struct list_head *pos, *npos;
 	struct use_case_verb *verb;
 
+	if (uc_mgr->local_config) {
+		snd_config_delete(uc_mgr->local_config);
+		uc_mgr->local_config = NULL;
+	}
+	if (uc_mgr->macros) {
+		snd_config_delete(uc_mgr->macros);
+		uc_mgr->macros = NULL;
+	}
 	list_for_each_safe(pos, npos, &uc_mgr->verb_list) {
 		verb = list_entry(pos, struct use_case_verb, list);
 		free(verb->name);
@@ -743,7 +776,6 @@ void uc_mgr_free_verb(snd_use_case_mgr_t *uc_mgr)
 
 void uc_mgr_free(snd_use_case_mgr_t *uc_mgr)
 {
-	snd_config_delete(uc_mgr->local_config);
 	uc_mgr_free_verb(uc_mgr);
 	uc_mgr_free_ctl_list(uc_mgr);
 	free(uc_mgr->card_name);
